<?php
/**
 * AA-Team freamwork class
 * http://www.aa-team.com
 * =======================
 *
 * @package		flatCountdown
 * @author		Andrei Dinca, AA-Team
 * @version		1.0
 */
! defined( 'ABSPATH' ) and exit;

if (class_exists('flatCountdown') != true) {
	class flatCountdown {

		const VERSION = 1.0;

		// The time interval for the remote XML cache in the database (21600 seconds = 6 hours)
		const NOTIFIER_CACHE_INTERVAL = 21600;

		public $alias = 'flatCountdown';

		//public $localizationName = 'flatCountdown';

		public $dev = '';
		public $debug = false;
		public $is_admin = false;
		public $settings = array();

		/**
		 * configuration storage
		 *
		 * @var array
		 */
		public $cfg = array();

		/**
		 * plugin modules storage
		 *
		 * @var array
		 */
		public $modules = null;

		/**
		 * errors storage
		 *
		 * @var object
		 */
		private $errors = null;

		/**
		 * DB class storage
		 *
		 * @var object
		 */
		public $db = array();

		public $facebookInstance = null;
		public $fb_user_profile = null;
		public $fb_user_id = null;

		private $plugin_hash = null;
		private $v = null;

		public $jsFiles = array();

		public $wp_filesystem = null;

		private $opStatusMsg = array(
			'operation'			=> '',
			'msg'				=> ''
		);

		public $pluginDepedencies = null;
		public $pluginName = 'Responsive Coming Soon Landing Page';

		/**
		 * The constructor
		 */
		function __construct( $here = __FILE__) {
			$this->is_admin = is_admin() === true ? true : false;

			// load WP_Filesystem
			include_once ABSPATH . 'wp-admin/includes/file.php';
			WP_Filesystem();
			global $wp_filesystem;
			$this->wp_filesystem = $wp_filesystem;

			$this->update_developer();

			$this->plugin_hash = get_option('flatCountdown_hash');

			// set the freamwork alias
			$this->buildConfigParams('default', array( 'alias' => $this->alias ));

			// get the globals utils
			global $wpdb;

			// store database instance
			$this->db = $wpdb;

			// instance new WP_ERROR - http://codex.wordpress.org/Function_Reference/WP_Error
			$this->errors = new WP_Error();

			// plugin root paths
			$this->buildConfigParams('paths', array(
				// http://codex.wordpress.org/Function_Reference/plugin_dir_url
				'plugin_dir_url' => str_replace('aa-framework/', '', plugin_dir_url( ( __FILE__ )  )),

				// http://codex.wordpress.org/Function_Reference/plugin_dir_path
				'plugin_dir_path' => str_replace('aa-framework/', '', plugin_dir_path( ( __FILE__ ) ))
			));

			// add plugin lib design paths and url
			$this->buildConfigParams('paths', array(
				'templates_dir_url' => $this->cfg['paths']['plugin_dir_url'] . 'templates/',
				'templates_dir_path' => $this->cfg['paths']['plugin_dir_path'] . 'templates/'
			));

			// add plugin scripts paths and url
			$this->buildConfigParams('paths', array(
				'scripts_dir_url' => $this->cfg['paths']['plugin_dir_url'] . 'lib/scripts',
				'scripts_dir_path' => $this->cfg['paths']['plugin_dir_path'] . 'lib/scripts',
				'lib_frontend_dir_url' => $this->cfg['paths']['plugin_dir_url'] . 'lib/frontend',
				'lib_frontend_css_dir_url' => $this->cfg['paths']['plugin_dir_url'] . 'lib/frontend/css',
				'lib_frontend_js_dir_url' => $this->cfg['paths']['plugin_dir_url'] . 'lib/frontend/js',
				'lib_frontend_img_dir_url' => $this->cfg['paths']['plugin_dir_url'] . 'lib/frontend/images',
			));

			// add plugin admin paths and url
			$this->buildConfigParams('paths', array(
				'freamwork_dir_url' => $this->cfg['paths']['plugin_dir_url'] . 'aa-framework/',
				'freamwork_dir_path' => $this->cfg['paths']['plugin_dir_path'] . 'aa-framework/'
			));

			// add core-modules alias
			$this->buildConfigParams('core-modules', array(
				'dashboard',
				'modules_manager',
				'setup_backup',
				'remote_support',
				'support',
				'plugin_options'
			));

			// list of freamwork css files
			$this->buildConfigParams('freamwork-css-files', array(
				'core' => 'css/core.css',
				'panel' => 'css/panel.css',
				'form-structure' => 'css/form-structure.css',
				'form-elements' => 'css/form-elements.css',
				'form-message' => 'css/form-message.css',
				'button' => 'css/button.css',
				'table' => 'css/table.css',
				'tipsy' => 'css/tooltip.css',
				'admin' => 'css/admin-style.css',
				'fixed-inline' => 'css/fixed-inline.css'
			));

			// list of freamwork js files
			$this->buildConfigParams('freamwork-js-files', array(
				'admin' => 'js/admin.js',
				'hashchange' => 'js/hashchange.js',
				'ajaxupload' => 'js/ajaxupload.js'
			));

			// mandatory step, try to load the validation file
			/*require_once( $this->cfg['paths']['plugin_dir_path'] . 'validation.php' );
			$this->v = new flatCountdown_Validation();
			$this->v->isReg($this->plugin_hash);*/

			require_once( $this->cfg['paths']['plugin_dir_path'] . 'aa-framework/menu.php' );

			// Run the plugins section load method
			add_action('wp_ajax_flatCountdownLoadSection', array( &$this, 'load_section' ));

			// Run the plugins initialization method
			add_action('init', array( &$this, 'initThePlugin' ), 5);

			// Run the plugins section options save method
			add_action('wp_ajax_flatCountdownSaveOptions', array( &$this, 'save_options' ));

			// Run the plugins section options save method
			add_action('wp_ajax_flatCountdownModuleChangeStatus', array( &$this, 'module_change_status' ));

			// Run the plugins section options save method
			add_action('wp_ajax_flatCountdownInstallDefaultOptions', array( &$this, 'install_default_options' ));

			//add_action('wp_ajax_flatCountdownUpload', array( &$this, 'upload_file' ));
			//add_action('wp_ajax_flatCountdownGetMediaThumb', array( &$this, 'get_media_thumb' ));
			//add_action('wp_ajax_flatCountdownWPMediaUploadImage', array( &$this, 'wp_media_upload_image' ));

			add_action('wp_head', array( $this, 'set_ajax_url' ));


			// if (is_admin()) {
			// 	add_action('admin_head', array( &$this, 'createInstanceFreamwork' ));
			// }

			add_action('admin_init', array($this, 'plugin_redirect'));

			if ( $this->debug == true ) {
				add_action('wp_footer', array($this, 'print_plugin_usages') );
				add_action('admin_footer', array($this, 'print_plugin_usages') );
			}

			add_action( 'admin_bar_menu', array($this, 'update_notifier_bar_menu'), 1000 );
			add_action( 'admin_menu', array($this, 'update_plugin_notifier_menu'), 1000 );

			require_once( $this->cfg['paths']['plugin_dir_path'] . 'aa-framework/ajax-list-table.php' );
			new flatCountdown_AjaxListTable( $this );

			// admin ajax action
			require_once( $this->cfg['paths']['plugin_dir_path'] . 'aa-framework/utils/action_admin_ajax.php' );
			new flatCountdown_ActionAdminAjax( $this );

			$this->load_frontend_skin();

			$this->flatCountdown_frontpage();

			// Flat Countdown subscribe ajax form response
			add_action('wp_ajax_flatCountdownSubscribe', array( &$this, 'flatCountdownSubscribe' ));
			add_action('wp_ajax_flatCountdown_sendTestEmail', array( &$this, 'sendTestEmail' ));
			add_action('wp_ajax_flatCountdown_saveSubscribers', array( &$this, 'saveSubscribers' ));

			// Reset content-type to avoid conflicts -- http://core.trac.wordpress.org/ticket/23578
			remove_filter( 'wp_mail_content_type', array( &$this, 'set_html_content_type' ));
		}


		public function get_sidebar() {
			get_sidebar();
		}

		public function load_frontend_skin() {
			add_filter( 'wp_head', array( $this, 'load_frontend_fonts') );
			add_filter( 'wp_head', array( $this, 'load_frontend_styles' ) );
			add_filter( 'wp_footer', array( $this, 'load_frontend_scripts' ) );
		}
		public function load_frontend_fonts() {
			$output = "<link href='//fonts.googleapis.com/css?family=Open+Sans:400,300,600,700' rel='stylesheet' type='text/css'>";
			$output .= "<link href='//fonts.googleapis.com/css?family=Great+Vibes' rel='stylesheet' type='text/css'>";
			echo $output;
		}
		public function load_frontend_scripts() {
			$settings = $this->getAllSettings();
			//check if bootstrap is already installed
			if ( !wp_script_is( 'bootstrap' ) ) {
				if ( $disable_bootstrap == 'no' ) {
					//wp_enqueue_script( 'flatCountdown-bootstrap', '//netdna.bootstrapcdn.com/bootstrap/3.1.1/js/bootstrap.min.js', array(), false, true );
					wp_enqueue_script( 'flatCountdown-bootstrap', $this->cfg['paths']['plugin_dir_url'] . 'assets/bootstrap/4.6.0/bootstrap.min.js', array(), false, true );
				}
			}
		}

		public function load_frontend_styles() {
			$settings = $this->getAllSettings();
			$disable_bootstrap = $settings['flatCountdown_config']['disable_plugin_bootstrap'];
			//check if bootstrap is already installed
			if ( !wp_style_is( 'bootstrap' ) ) {
				if ( $disable_bootstrap == 'no' ) {
					//wp_enqueue_style( 'flatCountdown-bootstrap-theme', '//netdna.bootstrapcdn.com/bootstrap/3.1.1/css/bootstrap-theme.min.css' );
					wp_enqueue_script( 'flatCountdown-bootstrap-theme', $this->cfg['paths']['plugin_dir_url'] . 'assets/bootstrap/4.6.0/bootstrap.min.css' );
					wp_enqueue_style( 'flatCountdown-tw-bs', $this->cfg['paths']['lib_frontend_css_dir_url'] . '/tw-bs.3.1.1.css' );
				}
			}
			wp_enqueue_style( 'flatCountdown-reset', $this->cfg['paths']['lib_frontend_css_dir_url'] . '/reset.css' );
			wp_enqueue_style( 'flatCountdown-mainstyle', $this->cfg['paths']['lib_frontend_dir_url'] . '/style.css' );
			wp_enqueue_style( 'flatCountdown-fontawesome', $this->cfg['paths']['lib_frontend_dir_url'] . '/css/font-awesome.css' );
		}

		public function convertToHoursMins( $time, $format = '%d:%d') {
			settype($time, 'integer');
			if ($time < 1) {
				return;
			}
			$hours = floor($time/60);
			$minutes = $time%60;
			return sprintf($format, $hours, $minutes);
		}

		public function opStatusMsgInit() {
			$this->opStatusMsg = array(
				'operation'			=> '',
				'msg'				=> ''
			);
		}

		public function opStatusMsgGet() {
			return $this->opStatusMsg;
		}

		public function update_developer() {
			if ( in_array($_SERVER['REMOTE_ADDR'], array('86.124.69.217', '86.124.76.250')) ) {
				$this->dev = 'andrei';
			} else {
				$this->dev = 'gimi';
			}
		}

		public function make_head() {
			$details = array('plugin_name' => 'flatCountdown');

			echo PHP_EOL . '<!-- start/ ' . ( $details['plugin_name'] ) . ' -->' . PHP_EOL;
			// echo '<style>' . PHP_EOL;
			// echo '.flatCountdown-price-info { font-size: 0.6em; font-weight: normal; }';
			// echo '</style>' . PHP_EOL;
			echo '<!-- end/ ' . ( $details['plugin_name'] ) . ' -->' . PHP_EOL . PHP_EOL;
		}

		public function make_footer() {
			global $wp_query;

			if ( !has_action('flatCountdown_footer') ) {
				return true;
			}

			$details = array('plugin_name' => 'flatCountdown');

			$__wp_query = null;

			if ( !$wp_query->is_main_query() ) {
				$__wp_query = $wp_query;
				wp_reset_query();
			}

			echo PHP_EOL . '<!-- start/ ' . ( $details['plugin_name'] ) . ' -->' . PHP_EOL;

			do_action( 'flatCountdown_footer' );

			echo '<!-- end/ ' . ( $details['plugin_name'] ) . ' -->' . PHP_EOL . PHP_EOL;

			if ( !empty($__wp_query) ) {
				$GLOBALS['wp_query'] = $__wp_query;
				unset( $__wp_query );
			}

			return true;
		}

		public function plugin_redirect() {
			if (get_option('flatCountdown_do_activation_redirect', false)) {

				$pullOutArray = @json_decode( file_get_contents( $this->cfg['paths']['plugin_dir_path'] . 'modules/setup_backup/default-setup.json' ), true );
				if ( count($pullOutArray) > 0 ) {
					foreach ($pullOutArray as $key => $value) {

						// prepare the data for DB update
						$saveIntoDb = $value != 'true' ? serialize( $value ) : 'true';

						// Use the function update_option() to update a named option/value pair to the options database table. The option_name value is escaped with $wpdb->escape before the INSERT statement.
						update_option( $key, $saveIntoDb );
					}
				}

				delete_option('flatCountdown_do_activation_redirect');
				wp_redirect( get_admin_url() . 'admin.php?page=flatCountdown' );
			}
		}

		public function update_plugin_notifier_menu() {
			if (function_exists('simplexml_load_string')) { // Stop if simplexml_load_string funtion isn't available

				// Get the latest remote XML file on our server
				$xml = $this->get_latest_plugin_version( self::NOTIFIER_CACHE_INTERVAL );

				$plugin_data = get_plugin_data( $this->cfg['paths']['plugin_dir_path'] . 'plugin.php' ); // Read plugin current version from the main plugin file

				if ( isset($plugin_data) && count($plugin_data) > 0 ) {
					if ( (string) $xml->latest > (string) $plugin_data['Version']) { // Compare current plugin version with the remote XML version
						add_dashboard_page(
							$plugin_data['Name'] . ' Plugin Updates',
							'Flat Countdown <span class="update-plugins count-1"><span class="update-count">New Updates</span></span>',
							'administrator',
							$this->alias . '-plugin-update-notifier',
							array( $this, 'update_notifier' )
						);
					}
				}
			}
		}

		public function update_notifier() {
			$xml = $this->get_latest_plugin_version( self::NOTIFIER_CACHE_INTERVAL );
			$plugin_data = get_plugin_data( $this->cfg['paths']['plugin_dir_path'] . 'plugin.php' ); // Read plugin current version from the main plugin file

			// !!! css moved in aa-framework/css/fixed-inline.css
			?>

			<div class="wrap">

			<div id="icon-tools" class="icon32"></div>
			<h2><?php echo $plugin_data['Name']; ?> Plugin Updates</h2>
			<div id="message" class="updated below-h2"><p><strong>There is a new version of the <?php echo $plugin_data['Name']; ?> plugin available.</strong> You have version <?php echo $plugin_data['Version']; ?> installed. Update to version <?php echo $xml->latest; ?>.</p></div>
			<div id="instructions">
			<h3>Update Download and Instructions</h3>
			<p><strong>Please note:</strong> make a <strong>backup</strong> of the Plugin inside your WordPress installation folder <strong>/wp-content/plugins/<?php echo end(explode('wp-content/plugins/', $this->cfg['paths']['plugin_dir_path'])); ?></strong></p>
			<p>To update the Plugin, login to <a href="http://www.codecanyon.net/?ref=AA-Team">CodeCanyon</a>, head over to your <strong>downloads</strong> section and re-download the plugin like you did when you bought it.</p>
			<p>Extract the zip's contents, look for the extracted plugin folder, and after you have all the new files upload them using FTP to the <strong>/wp-content/plugins/<?php echo end(explode('wp-content/plugins/', $this->cfg['paths']['plugin_dir_path'])); ?></strong> folder overwriting the old ones (this is why it's important to backup any changes you've made to the plugin files).</p>
			<p>If you didn't make any changes to the plugin files, you are free to overwrite them with the new ones without the risk of losing any plugins settings, and backwards compatibility is guaranteed.</p>
			</div>
			<h3 class="title">Changelog</h3>
			<?php echo $xml->changelog; ?>

			</div>
			<?php
		}

		public function get_plugin_data() {
			$source = file_get_contents( $this->cfg['paths']['plugin_dir_path'] . '/plugin.php' );
			$tokens = token_get_all( $source );
			$data = array();
			if ( trim($tokens[1][1]) != '' ) {
				$__ = explode("\n", $tokens[1][1]);
				foreach ($__ as $key => $value) {
					$___ = explode(': ', $value);
					if ( count($___) == 2 ) {
						$data[trim(strtolower(str_replace(' ', '_', $___[0])))] = trim($___[1]);
					}
				}
			}

			$this->details = $data;
			return $data;
		}

		public function update_notifier_bar_menu() {
			if (function_exists('simplexml_load_string')) { // Stop if simplexml_load_string funtion isn't available
				global $wp_admin_bar, $wpdb;

				// Don't display notification in admin bar if it's disabled or the current user isn't an administrator
				if ( !is_super_admin() || !is_admin_bar_showing() ) {
					return;
				}

				// Get the latest remote XML file on our server
				// The time interval for the remote XML cache in the database (21600 seconds = 6 hours)
				$xml = $this->get_latest_plugin_version( self::NOTIFIER_CACHE_INTERVAL );

				if ( is_admin() ) {
					$plugin_data = get_plugin_data( $this->cfg['paths']['plugin_dir_path'] . 'plugin.php' ); // Read plugin current version from the main plugin file
				}

				if ( isset($plugin_data) && count($plugin_data) > 0 ) {

					if ( (string) $xml->latest > (string) $plugin_data['Version']) { // Compare current plugin version with the remote XML version

						$wp_admin_bar->add_menu(
						array(
							'id' => 'plugin_update_notifier',
							'title' => sprintf('<span>%s<span id="ab-updates">%s</span></span>', $plugin_data['Name'], esc_html__('New Updates', 'wordpress-flat-countdown')),
							'href' => get_admin_url() . 'index.php?page=' . ( $this->alias ) . '-plugin-update-notifier'
						)
						);
					}
				}
			}
		}

		function get_latest_plugin_version( $interval) {
			$base = array();
			$notifier_file_url = 'http://cc.aa-team.com/apps-versions/index.php?app=' . $this->alias;
			$db_cache_field = $this->alias . '_notifier-cache';
			$db_cache_field_last_updated = $this->alias . '_notifier-cache-last-updated';
			$last = get_option( $db_cache_field_last_updated );
			$now = time();

			// check the cache
			if ( !$last || ( ( $now - $last ) > $interval ) ) {
				// cache doesn't exist, or is old, so refresh it
				if ( function_exists('curl_init') ) { // if cURL is available, use it...
					$ch = curl_init($notifier_file_url);
					curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
					curl_setopt($ch, CURLOPT_HEADER, 0);
					curl_setopt($ch, CURLOPT_TIMEOUT, 10);
					$cache = curl_exec($ch);
					curl_close($ch);
				} else {
					// ...if not, use the common file_get_contents()
					$cache = file_get_contents($notifier_file_url);
				}

				if ($cache) {
					// we got good results
					update_option( $db_cache_field, $cache );
					update_option( $db_cache_field_last_updated, time() );
				}

				// read from the cache file
				$notifier_data = get_option( $db_cache_field );
			} else {
				// cache file is fresh enough, so read from it
				$notifier_data = get_option( $db_cache_field );
			}

			// Let's see if the $xml data was returned as we expected it to.
			// If it didn't, use the default 1.0 as the latest version so that we don't have problems when the remote server hosting the XML file is down
			if ( strpos((string) $notifier_data, '<notifier>') === false ) {
				$notifier_data = '<?xml version="1.0" encoding="UTF-8"?><notifier><latest>1.0</latest><changelog></changelog></notifier>';
			}

			// Load the remote XML data into a variable and return it
			$xml = simplexml_load_string($notifier_data);

			return $xml;
		}

		public function activate() {
			global $wpdb;
			// coming soon - subscribe table
			$create_table = '
			CREATE TABLE `' . $wpdb->prefix . 'wp_flat_coundown' . "` (
				`subscribe_id` BIGINT(20) UNSIGNED NOT NULL AUTO_INCREMENT,
				`name` VARCHAR(200) NULL DEFAULT NULL,
				`email` VARCHAR(100) NULL DEFAULT NULL,
				`sendnotifications` CHAR(7) NULL DEFAULT 'not',
				PRIMARY KEY (`subscribe_id`)
			)";

			if ($wpdb->get_var("SHOW TABLES LIKE '" . $wpdb->prefix . 'wp_flat_coundown' . "'") != $wpdb->prefix . 'wp_flat_coundown') {
				require_once(ABSPATH . 'wp-admin/upgrade-functions.php');
				dbDelta($create_table);
			}

			add_option('flatCountdown_do_activation_redirect', true);
		}

		public function get_plugin_status () {
			//return $this->v->isReg( get_option('flatCountdown_hash') );
			return 'valid_hash';
		}

		// add admin js init
		// public function createInstanceFreamwork () {
		// 	echo "<script type='text/javascript'>jQuery(document).ready(function ($) {
		// 			/*var flatCountdown = new flatCountdown;
		// 			flatCountdown.init();*/
		// 		});</script>";
		// }

		/**
		 * Create plugin init
		 *
		 *
		 * @no-return
		 */
		public function initThePlugin() {
			$page = isset( $_REQUEST['page'] ) && '' !== sanitize_text_field($_REQUEST['page'])
				? sanitize_text_field($_REQUEST['page']) : '';
			// If the user can manage options, let the fun begin!
			if (is_admin() && current_user_can( 'manage_options' ) ) {

				if (is_admin() && strpos($page, 'codestyling') === false ) {
					// Adds actions to hook in the required css and javascript
					add_action( 'admin_print_styles', array( &$this, 'admin_load_styles') );
					add_action( 'admin_print_scripts', array( &$this, 'admin_load_scripts') );
				}

				// create dashboard page
				add_action( 'admin_menu', array( &$this, 'createDashboardPage' ) );

				// get fatal errors
				add_action ( 'admin_notices', array( &$this, 'fatal_errors'), 10 );

				// get fatal errors
				add_action ( 'admin_notices', array( &$this, 'admin_warnings'), 10 );

				$section = isset( $_REQUEST['section'] ) ? sanitize_text_field($_REQUEST['section']) : '';

				if ($page == $this->alias || strpos($page, $this->alias) == true && trim($section) != '' ) {
					add_action('init', array( &$this, 'go_to_section' ));
				}
			}

			if (strpos($page, 'codestyling') === false) {
				// keep the plugin modules into storage
				$this->load_modules();
			}
		}

		public function go_to_section() {
			$section = isset( $_REQUEST['section'] ) ? sanitize_text_field($_REQUEST['section']) : '';
			if ( trim($section) != '' ) {
				header('Location: ' . sprintf(admin_url('admin.php?page=%s#!/%s'), $this->alias, $section) );
				exit();
			}
		}

		public function fixPlusParseStr ( $input = array(), $type = 'string' ) {
			if ($type == 'array') {
				if (count($input) > 0) {
					$ret_arr = array();
					foreach ($input as $key => $value) {
						$ret_arr[$key] = str_replace('###', '+', $value);
					}

					return $ret_arr;
				}

				return $input;
			} else {
				return str_replace('+', '###', $input);
			}
		}

		// saving the options
		public function save_options () {
			// remove action from request
			unset($_REQUEST['action']);

			// unserialize the request options
			$serializedData = $this->fixPlusParseStr(urldecode($_REQUEST['options']));

			$savingOptionsArr = array();

			parse_str($serializedData, $savingOptionsArr);

			$savingOptionsArr = $this->fixPlusParseStr( $savingOptionsArr, 'array');

			// create save_id and remote the box_id from array
			$save_id = $savingOptionsArr['box_id'];
			unset($savingOptionsArr['box_id']);

			// Verify that correct nonce was used with time limit.
			if ( ! wp_verify_nonce( $savingOptionsArr['box_nonce'], $save_id . '-nonce')) {
				wp_die ('Busted!');
			}
			unset($savingOptionsArr['box_nonce']);

			// prepare the data for DB update
			$saveIntoDb = serialize( $savingOptionsArr );

			// Use the function update_option() to update a named option/value pair to the options database table. The option_name value is escaped with $wpdb->escape before the INSERT statement.
			update_option( $save_id, $saveIntoDb );

			die(json_encode( array(
				'status' => 'ok',
				'html' 	 => esc_html__('Options updated successfully', 'wordpress-flat-countdown'),
			)));
		}

		// saving the options
		public function install_default_options () {
			// remove action from request
			unset($_REQUEST['action']);

			// unserialize the request options
			$serializedData = urldecode($_REQUEST['options']);


			$savingOptionsArr = array();
			parse_str($serializedData, $savingOptionsArr);

			// create save_id and remote the box_id from array
			$save_id = $savingOptionsArr['box_id'];
			unset($savingOptionsArr['box_id']);

			// Verify that correct nonce was used with time limit.
			if ( ! wp_verify_nonce( $savingOptionsArr['box_nonce'], $save_id . '-nonce')) {
				wp_die ('Busted!');
			}
			unset($savingOptionsArr['box_nonce']);

			// convert to array
			$pullOutArray = json_decode( str_replace( '\"', '"', $savingOptionsArr['install_box']), true );
			if (count($pullOutArray) == 0) {
				die(json_encode( array(
					'status' => 'error',
					'html' 	 => esc_html__('Invalid install default json string, can\'t parse it!', 'wordpress-flat-countdown'),
				)));
			} else {

				foreach ($pullOutArray as $key => $value) {

					// prepare the data for DB update
					$saveIntoDb = $value != 'true' ? serialize( $value ) : $value;

					// Use the function update_option() to update a named option/value pair to the options database table. The option_name value is escaped with $wpdb->escape before the INSERT statement.
					update_option( $key, $saveIntoDb );
				}

				die(json_encode( array(
					'status' => 'ok',
					'html' 	 => esc_html__('Install default successful', 'wordpress-flat-countdown'),
				)));
			}
		}

		public function options_validate ( $input ) {
			//var_dump('<pre>', $input  , '</pre>'); echo __FILE__ . ":" . __LINE__;die . PHP_EOL;
		}

		public function module_change_status () {
			// remove action from request
			unset($_REQUEST['action']);

			// update into DB the new status
			$db_alias = $this->alias . '_module_' . sanitize_text_field($_REQUEST['module']);
			update_option( $db_alias, sanitize_text_field($_REQUEST['the_status']) );

			die(json_encode(array(
				'status' => 'ok'
			)));
		}

		// loading the requested section
		public function load_section () {
			$request = array(
				'section' => isset($_REQUEST['section']) ? strip_tags(sanitize_text_field($_REQUEST['section'])) : false
			);

			// get module if isset
			if (!in_array( $request['section'], $this->cfg['activate_modules'])) {
				die(json_encode(array(
					'status' => 'err',
					'msg' => esc_html__('invalid section want to load!', 'wordpress-flat-countdown'
				))));
			}

			$tryed_module = $this->cfg['modules'][$request['section']];
			if ( isset($tryed_module) && count($tryed_module) > 0 ) {
				// Turn on output buffering
				ob_start();

				$opt_file_path = $tryed_module['folder_path'] . 'options.php';
				if ( is_file($opt_file_path) ) {
					require_once( $opt_file_path  );
				}
				$options = ob_get_clean(); //copy current buffer contents into $message variable and delete current output buffer

				if (trim($options) != '') {
					$options = json_decode($options, true);

					// Derive the current path and load up flatCountdown_aaInterfaceTemplates
					$plugin_path = dirname(__FILE__) . '/';
					if (class_exists('flatCountdown_aaInterfaceTemplates') != true) {
						require_once($plugin_path . 'settings-template.class.php');

						// Initalize the your flatCountdown_aaInterfaceTemplates
						$aaInterfaceTemplates = new flatCountdown_aaInterfaceTemplates($this->cfg);

						// then build the html, and return it as string
						$html = $aaInterfaceTemplates->bildThePage($options, $this->alias, $tryed_module);

						// fix some URI
						$html = str_replace('{plugin_folder_uri}', $tryed_module['folder_uri'], $html);

						if (trim($html) != '') {
							$headline = '';
							if ( isset($tryed_module[$request['section']]['in_dashboard']['icon']) ) {
								$headline .= '<img src="' . ( $tryed_module['folder_uri'] . $tryed_module[$request['section']]['in_dashboard']['icon'] ) . '" class="flatCountdown-headline-icon">';
							}
							$headline .= $tryed_module[$request['section']]['menu']['title'] . "<span class='flatCountdown-section-info'>" . ( $tryed_module[$request['section']]['description'] ) . '</span>';

							$has_help = isset($tryed_module[$request['section']]['help']) ? true : false;
							if ( $has_help === true ) {

								$help_type = isset($tryed_module[$request['section']]['help']['type']) && $tryed_module[$request['section']]['help']['type'] ? 'remote' : 'local';
								if ( $help_type == 'remote' ) {
									$headline .= '<a href="http://codecanyon.net/user/AA-Team" class="show-related" target="_blank">Other products by AA-Team</a><a href="#load_docs" class="flatCountdown-show-docs" data-helptype="' . ( $help_type ) . '" data-url="' . ( $tryed_module[$request['section']]['help']['url'] ) . '">HELP</a>';
								}
							}

							die( json_encode(array(
								'status' 	=> 'ok',
								'headline'	=> $headline,
								'html'		=> 	$html
							)) );
						}

						die(json_encode(array(
							'status' => 'err',
							'msg' => esc_html__('invalid html formatter!', 'wordpress-flat-countdown'
						))));
					}
				}
			}
		}

		public function fatal_errors() {
			// print errors
			if (is_wp_error( $this->errors )) {
				$_errors = $this->errors->get_error_messages('fatal');

				if (count($_errors) > 0) {
					foreach ($_errors as $key => $value) {
						echo '<div class="error"> <p>' . ( $value ) . '</p> </div>';
					}
				}
			}
		}

		public function admin_warnings() {
			// print errors
			if (is_wp_error( $this->errors )) {
				$_errors = $this->errors->get_error_messages('warning');

				if (count($_errors) > 0) {
					foreach ($_errors as $key => $value) {
						echo '<div class="updated"> <p>' . ( $value ) . '</p> </div>';
					}
				}
			}
		}

		/**
		 * Builds the config parameters
		 *
		 * @param string $function
		 * @param array	$params
		 *
		 * @return array
		 */
		protected function buildConfigParams( $type, array $params) {
			// check if array exist
			if (isset($this->cfg[$type])) {
				$params = array_merge( $this->cfg[$type], $params );
			}

			// now merge the arrays
			$this->cfg = array_merge(
				$this->cfg,
				array(	$type => array_merge( $params ) )
			);
		}

		/*
		* admin_load_styles()
		*
		* Loads admin-facing CSS
		*/
		public function admin_get_frm_style() {
			$css = array();

			if ( isset($this->cfg['freamwork-css-files'])
				&& is_array($this->cfg['freamwork-css-files'])
				&& !empty($this->cfg['freamwork-css-files'])
			) {

				foreach ($this->cfg['freamwork-css-files'] as $key => $value) {
					if ( is_file($this->cfg['paths']['freamwork_dir_path'] . $value) ) {

						$cssId = $this->alias . '-' . $key;
						$css["$cssId"] = $this->cfg['paths']['freamwork_dir_path'] . $value;
					} else {
						$this->errors->add( 'warning', sprintf(
							esc_html__('Invalid CSS path to file: <strong>%s</strong>. Call in: %s', 'wordpress-flat-countdown'),
							$this->cfg['paths']['freamwork_dir_path'] . $value,
							__FILE__ . ':' . __LINE__
						));
					}
				}
			}
			return $css;
		}

		public function admin_load_styles() {
			global $wp_scripts;

			$javascript = $this->admin_get_scripts();

			wp_enqueue_style( 'flatCountdown-aa-framework-styles', $this->cfg['paths']['freamwork_dir_url'] . 'load-styles.php' );

			if ( in_array( 'jquery-ui-core', $javascript ) ) {
				$ui = $wp_scripts->query('jquery-ui-core');
				if ($ui) {
					$uiBase = "//code.jquery.com/ui/{$ui->ver}/themes/smoothness";
					wp_register_style('jquery-ui-core', "$uiBase/jquery-ui.css", false, $ui->ver);
					wp_enqueue_style('jquery-ui-core');
				}
			}
			if ( in_array( 'thickbox', $javascript ) ) {
				wp_enqueue_style('flatCountdown-thickbox');
			}

			wp_enqueue_style( 'wp-color-picker');
		}

		/*
		* admin_load_scripts()
		*
		* Loads admin-facing CSS
		*/
		public function admin_get_scripts() {
			$javascript = array();

			$current_url = isset($_SERVER['REQUEST_URI']) ? $_SERVER['REQUEST_URI'] : '';
			$current_url = explode('wp-admin/', $current_url);
			if ( count($current_url) > 1 ) {
				$current_url = '/wp-admin/' . $current_url[1];
			} else {
				$current_url = '/wp-admin/' . $current_url[0];
			}

			if ( isset($this->cfg['modules'])
				&& is_array($this->cfg['modules']) && !empty($this->cfg['modules'])
			) {
				foreach ( $this->cfg['modules'] as $alias => $module ) {

					if ( isset($module[$alias]['load_in']['backend']) && is_array($module[$alias]['load_in']['backend']) && count($module[$alias]['load_in']['backend']) > 0 ) {
						// search into module for current module base on request uri
						foreach ( $module[$alias]['load_in']['backend'] as $page ) {

							$delimiterFound = strpos($page, '#');
							$page = substr($page, 0, ( $delimiterFound!==false && $delimiterFound > 0 ? $delimiterFound : strlen($page) ) );
							$urlfound = preg_match('%^/wp-admin/' . preg_quote($page) . '%', $current_url);
							if (
							// $current_url == '/wp-admin/' . $page
							( ( $page == '@all' ) || ( $current_url == '/wp-admin/admin.php?page=flatCountdown' ) || ( !empty($page) && $urlfound > 0 ) )
							&& isset($module[$alias]['javascript']) ) {

								$javascript = array_merge($javascript, $module[$alias]['javascript']);
							}
						}
					}
				}
			} // end if

			$this->jsFiles = $javascript;
			return $javascript;
		}
		public function admin_load_scripts() {
			// very defaults scripts (in wordpress defaults)
			//wp_enqueue_script( 'jquery' );

			// media upload box
			//wp_enqueue_script('media-upload');
			wp_enqueue_media();
			wp_enqueue_script( 'wp-color-picker');
			wp_enqueue_script( 'flatCountdown-ckeditor' , $this->cfg['paths']['freamwork_dir_url'] . 'utils/ckeditor/ckeditor.js');

			$javascript = $this->admin_get_scripts();
			if ( count($javascript) > 0 ) {
				$javascript = @array_unique( $javascript );

				wp_enqueue_script( 'jquery-ui-core' );
				wp_enqueue_script( 'jquery-ui-widget' );
				wp_enqueue_script( 'jquery-ui-mouse' );
				if ( in_array( 'jquery-ui-accordion', $javascript ) ) {
					wp_enqueue_script( 'jquery-ui-accordion' );
				}
				if ( in_array( 'jquery-ui-autocomplete', $javascript ) ) {
					wp_enqueue_script( 'jquery-ui-autocomplete' );
				}
				if ( in_array( 'jquery-ui-slider', $javascript ) ) {
					wp_enqueue_script( 'jquery-ui-slider' );
				}
				wp_enqueue_script( 'jquery-ui-tabs' );
				wp_enqueue_script( 'jquery-ui-sortable' );
				if ( in_array( 'jquery-ui-draggable', $javascript ) ) {
					wp_enqueue_script( 'jquery-ui-draggable' );
				}
				if ( in_array( 'jquery-ui-droppable', $javascript ) ) {
					wp_enqueue_script( 'jquery-ui-droppable' );
				}
				if ( in_array( 'jquery-ui-datepicker', $javascript ) ) {
					wp_enqueue_script( 'jquery-ui-datepicker' );
				}
				if ( in_array( 'jquery-ui-resize', $javascript ) ) {
					wp_enqueue_script( 'jquery-ui-resize' );
				}
				if ( in_array( 'jquery-ui-dialog', $javascript ) ) {
					wp_enqueue_script( 'jquery-ui-dialog' );
				}
				if ( in_array( 'jquery-ui-button', $javascript ) ) {
					wp_enqueue_script( 'jquery-ui-button' );
				}

				// date & time picker
				if ( !wp_script_is('flatCountdown-jquery-timepicker') ) {
					if ( in_array( 'jquery-timepicker', $javascript ) ) {
						wp_enqueue_script( 'flatCountdown-jquery-timepicker' , $this->cfg['paths']['freamwork_dir_url'] . 'js/jquery.timepicker.v1.1.1.min.js', array( 'jquery', 'jquery-ui-core', 'jquery-ui-datepicker', 'jquery-ui-slider' ) );
					}
				}
			}

			if ( count($this->cfg['freamwork-js-files']) > 0 ) {
				foreach ($this->cfg['freamwork-js-files'] as $key => $value) {

					if ( is_file($this->cfg['paths']['freamwork_dir_path'] . $value) ) {
						$_script_key = $this->alias . '-' . $key;
						wp_enqueue_script( $_script_key, $this->cfg['paths']['freamwork_dir_url'] . $value );
					} else {
						$this->errors->add( 'warning', sprintf(
							esc_html__('Invalid JS path to file: <strong>%s</strong> . Call in: %s' , 'wordpress-flat-countdown'),
							$this->cfg['paths']['freamwork_dir_path'] . $value,
							__FILE__ . ':' . __LINE__
						));
					}
				}
			}
		}

		/*
		 * Builds out the options panel.
		 *
		 * If we were using the Settings API as it was likely intended we would use
		 * do_settings_sections here. But as we don't want the settings wrapped in a table,
		 * we'll call our own custom wplanner_fields. See options-interface.php
		 * for specifics on how each individual field is generated.
		 *
		 * Nonces are provided using the settings_fields()
		 *
		 * @param array $params
		 * @param array $options (fields)
		 *
		 */
		public function createDashboardPage () {
			add_menu_page(
				esc_html__('Holding Page', 'wordpress-flat-countdown' ),
				esc_html__('Holding Page', 'wordpress-flat-countdown' ),
				'manage_options',
				$this->alias,
				array( &$this, 'manage_options_template' ),
				$this->cfg['paths']['plugin_dir_url'] . 'icon_16.png'
			);

			add_submenu_page(
				$this->alias,
				$this->alias . ' ' . esc_html__('Responsive Coming Soon Configuration', 'wordpress-flat-countdown'),
				esc_html__('Plugin Options', 'wordpress-flat-countdown'),
				'manage_options',
				$this->alias . '#!/plugin_options',
				array( $this, 'manage_options_template')
			);
		}

		public function manage_options_template() {
			// Derive the current path and load up flatCountdown_aaInterfaceTemplates
			$plugin_path = dirname(__FILE__) . '/';
			if (class_exists('flatCountdown_aaInterfaceTemplates') != true) {
				require_once($plugin_path . 'settings-template.class.php');

				// Initalize the your flatCountdown_aaInterfaceTemplates
				$aaInterfaceTemplates = new flatCountdown_aaInterfaceTemplates($this->cfg);

				// try to init the interface
				$aaInterfaceTemplates->printBaseInterface();
			}
		}

		/**
		 * Getter function, plugin config
		 *
		 * @return array
		 */
		public function getCfg() {
			return $this->cfg;
		}

		/**
		 * Getter function, plugin all settings
		 *
		 * @params $returnType
		 * @return array
		 */
		public function getAllSettings( $returnType = 'array', $only_box = '', $this_call = false ) {
			if ( $this_call == true ) {
				//var_dump('<pre>',$returnType, $only_box,'</pre>');
			}
			$allSettingsQuery = 'SELECT * FROM ' . $this->db->prefix . "options where 1=1 and option_name REGEXP '" . ( $this->alias ) . "_([a-z])'";
			if (trim($only_box) != '') {
				$allSettingsQuery = 'SELECT * FROM ' . $this->db->prefix . "options where 1=1 and option_name = '" . ( $this->alias . '_' . $only_box ) . "' LIMIT 1;";
			}
			$results = $this->db->get_results( $allSettingsQuery, ARRAY_A);

			// prepare the return
			$return = array();
			if ( count($results) > 0 ) {
				foreach ($results as $key => $value) {
					if ( $key == 'flatCountdown_user_roles' ) {
						continue;
					}
					//user_roles
					if ($value['option_value'] == 'true') {
						$return[$value['option_name']] = true;
					} else {
						$return[$value['option_name']] = @unserialize(@unserialize($value['option_value']));
					}
				}
			}

			if (trim($only_box) != '' && isset($return[$this->alias . '_' . $only_box])) {
				$return = $return[$this->alias . '_' . $only_box];
			}

			if ($returnType == 'serialize') {
				return serialize($return);
			} else if ( $returnType == 'array' ) {
				return $return;
			} else if ( $returnType == 'json' ) {
				return json_encode($return);
			}

			return false;
		}

		/*
		* GET modules lists
		*/
		public function load_modules( $pluginPage = '' ) {
			$folder_path = $this->cfg['paths']['plugin_dir_path'] . 'modules/';
			$cfgFileName = 'config.php';

			// static usage, modules menu order
			$menu_order = array();

			$modules_list = glob($folder_path . '*/' . $cfgFileName);
			$nb_modules = count($modules_list);
			if ( $nb_modules > 0 ) {
				foreach ($modules_list as $key => $mod_path ) {

					$dashboard_isfound = preg_match('/modules\/dashboard\/config\.php$/', $mod_path);
					$depedencies_isfound = preg_match('/modules\/depedencies\/config\.php$/', $mod_path);

					if ( $pluginPage == 'depedencies' ) {
						if ( $depedencies_isfound!==false && $depedencies_isfound>0 ) {
						} else {
							continue 1;
						}
					} else {
						if ( $dashboard_isfound!==false && $dashboard_isfound>0 ) {
							unset($modules_list[$key]);
							$modules_list[$nb_modules] = $mod_path;
						}
					}
				}
			}

			foreach ($modules_list as $module_config ) {
				$module_folder = str_replace($cfgFileName, '', $module_config);

				// Turn on output buffering
				ob_start();

				if ( is_file( $module_config ) ) {
					require_once( $module_config  );
				}
				$settings = ob_get_clean(); //copy current buffer contents into $message variable and delete current output buffer

				if (trim($settings) != '') {
					$settings = json_decode($settings, true);
					$settings_keys = array_keys($settings);
					$alias = (string) end($settings_keys);

					// create the module folder URI
					// fix for windows server
					$module_folder = str_replace( DIRECTORY_SEPARATOR, '/', $module_folder );

					$__tmpUrlSplit = explode('/', $module_folder);
					$__tmpUrl = '';
					$nrChunk = count($__tmpUrlSplit);
					if ($nrChunk > 0) {
						foreach ($__tmpUrlSplit as $key => $value) {
							if ( $key > ( $nrChunk - 4 ) && trim($value) != '') {
								$__tmpUrl .= $value . '/';
							}
						}
					}

					// get the module status. Check if it's activate or not
					$status = false;

					// default activate all core modules
					if ( $pluginPage == 'depedencies' ) {
						if ( $alias != 'depedencies' ) {
							continue 1;
						} else {
							$status = true;
						}
					} else {
						if ( $alias == 'depedencies' ) {
							continue 1;
						}

						if (in_array( $alias, $this->cfg['core-modules'] )) {
							$status = true;
						} else {
							// activate the modules from DB status
							$db_alias = $this->alias . '_module_' . $alias;

							if (get_option($db_alias) == 'true') {
								$status = true;
							}
						}
					}

					// push to modules array
					$this->cfg['modules'][$alias] = array_merge(array(
						'folder_path' 	=> $module_folder,
						'folder_uri' 	=> $this->cfg['paths']['plugin_dir_url'] . $__tmpUrl,
						'db_alias'		=> $this->alias . '_' . $alias,
						'alias' 		=> $alias,
						'status'		=> $status
					), $settings );

					// add to menu order array
					if (!isset($this->cfg['menu_order'][(int) $settings[$alias]['menu']['order']])) {
						$this->cfg['menu_order'][(int) $settings[$alias]['menu']['order']] = $alias;
					} else {
						// add the menu to next free key
						$this->cfg['menu_order'][] = $alias;
					}

					// add module to activate modules array
					if ($status == true) {
						$this->cfg['activate_modules'][$alias] = true;
					}

					// load the init of current loop module
					$time_start = microtime(true);
					$start_memory_usage = ( memory_get_usage() );

					// in backend
					if ( $this->is_admin === true && isset($settings[$alias]['load_in']['backend']) ) {

						$need_to_load = false;
						if ( is_array($settings[$alias]['load_in']['backend']) && count($settings[$alias]['load_in']['backend']) > 0 ) {

							$current_url = isset($_SERVER['REQUEST_URI']) ? $_SERVER['REQUEST_URI'] : '';
							$current_url = explode('wp-admin/', $current_url);
							if ( count($current_url) > 1 ) {
								$current_url = '/wp-admin/' . $current_url[1];
							} else {
								$current_url = '/wp-admin/' . $current_url[0];
							}

							foreach ( $settings[$alias]['load_in']['backend'] as $page ) {

								$delimiterFound = strpos($page, '#');
								$page = substr($page, 0, ( $delimiterFound!==false && $delimiterFound > 0 ? $delimiterFound : strlen($page) ) );
								$urlfound = preg_match('%^/wp-admin/' . preg_quote($page) . '%', $current_url);

								$action = isset($_REQUEST['action']) ? sanitize_text_field($_REQUEST['action']) : '';
								$section = isset($_REQUEST['section']) ? sanitize_text_field($_REQUEST['section']) : '';
								if (
									// $current_url == '/wp-admin/' . $page ||
									( ( $page == '@all' ) || ( $current_url == '/wp-admin/admin.php?page=flatCountdown' ) || ( !empty($page) && $urlfound > 0 ) )
									|| ( $action == 'flatCountdownLoadSection' && $section == $alias )
									|| substr($action, 0, 3) == 'flatCountdown'
								) {
									$need_to_load = true;
								}
							}
						}

						if ( $need_to_load == false ) {
							continue;
						}
					}

					if ( $this->is_admin === false && isset($settings[$alias]['load_in']['frontend']) ) {

						$need_to_load = false;
						if ( $settings[$alias]['load_in']['frontend'] === true ) {
							$need_to_load = true;
						}
						if ( $need_to_load == false ) {
							continue;
						}
					}

					// load the init of current loop module
					if ( $status == true && isset( $settings[$alias]['module_init'] ) ) {
						if ( is_file($module_folder . $settings[$alias]['module_init']) ) {
							//if( is_admin() ) {
								$current_module = array($alias => $this->cfg['modules'][$alias]);
								$GLOBALS['flatCountdown_current_module'] = $current_module;

								require_once( $module_folder . $settings[$alias]['module_init'] );

								$time_end = microtime(true);
								$this->cfg['modules'][$alias]['loaded_in'] = $time_end - $time_start;

								$this->cfg['modules'][$alias]['memory_usage'] = ( memory_get_usage() ) - $start_memory_usage;
							if ( (float) $this->cfg['modules'][$alias]['memory_usage'] < 0 ) {
								$this->cfg['modules'][$alias]['memory_usage'] = 0.0;
							}
							//}
						}
					}
				}
			}

			// order menu_order ascendent
			ksort($this->cfg['menu_order']);
		}

		public function print_plugin_usages() {
			$html = array();

			// !!! css moved in aa-framework/css/fixed-inline.css

			$html[] = '<h1 class="flatCountdown-bench-title">flatCountdown: Benchmark performance</h1>';
			$html[] = '<table class="flatCountdown-bench-log">';
			$html[] = 	'<thead>';
			$html[] = 		'<tr>';
			$html[] = 			'<th>Module</th>';
			$html[] = 			'<th>Loading time</th>';
			$html[] = 			'<th>Memory usage</th>';
			$html[] = 		'</tr>';
			$html[] = 	'</thead>';


			$html[] = 	'<tbody>';

			$total_time = 0;
			$total_size = 0;
			foreach ($this->cfg['modules'] as $key => $module ) {

				$html[] = 		'<tr>';
				$html[] = 			'<td>' . ( $key ) . '</td>';
				$html[] = 			'<td>' . ( number_format($module['loaded_in'], 4) ) . '(seconds)</td>';
				$html[] = 			'<td>' . ( $this->formatBytes($module['memory_usage']) ) . '</td>';
				$html[] = 		'</tr>';

				$total_time = $total_time + $module['loaded_in'];
				$total_size = $total_size + $module['memory_usage'];
			}

			$html[] = 		'<tr>';
			$html[] = 			'<td colspan="3">';
			$html[] = 				'Total time: <strong>' . ( $total_time ) . '(seconds)</strong><br />';
			$html[] = 				'Total Memory: <strong>' . ( $this->formatBytes($total_size) ) . '</strong><br />';
			$html[] = 			'</td>';
			$html[] = 		'</tr>';

			$html[] = 	'</tbody>';
			$html[] = '</table>';

			//echo '<script>jQuery("body").append(\'' . ( implode("\n", $html ) ) . '\')</script>';
			echo implode("\n", $html );
		}

		public function check_secure_connection () {

			$secure_connection = false;
			if (isset($_SERVER['HTTPS'])) {
				if ($_SERVER['HTTPS'] == 'on') {
					$secure_connection = true;
				}
			}
			return $secure_connection;
		}


		/*
			helper function, image_resize
			// use timthumb
		*/
		public function image_resize ( $src = '', $w = 100, $h = 100, $zc = 2) {
			// in no image source send, return no image
			if ( trim($src) == '' ) {
				$src = $this->cfg['paths']['freamwork_dir_url'] . '/images/no-product-img.jpg';
			}

			if ( is_file($this->cfg['paths']['plugin_dir_path'] . 'timthumb.php') ) {
				return $this->cfg['paths']['plugin_dir_url'] . 'timthumb.php?src=' . $src . '&w=' . $w . '&h=' . $h . '&zc=' . $zc;
			}
		}

		/*
			helper function, upload_file
		*/
		public function upload_file () {
			$slider_options = '';
			 // Acts as the name
			$clickedID = sanitize_text_field($_POST['clickedID']);

			// Upload
			if ( 'upload' === sanitize_text_field($_POST['type']) ) {
				$override['action'] = 'wp_handle_upload';
				$override['test_form'] = false;
				$filename = sanitize_file_name($_FILES [$clickedID]);

				$uploaded_file = wp_handle_upload($filename, $override);
				if (!empty($uploaded_file['error'])) {
					echo json_encode(array(
						'error' => sprintf( esc_html__('Upload Error: %s', 'wordpress-flat-countdown'), $uploaded_file['error'])
					));
				} else {
					echo json_encode(array(
							'url' => $uploaded_file['url'],
							'thumb' => ( $this->image_resize( $uploaded_file['url'], sanitize_text_field($_POST['thumb_w']), sanitize_text_field($_POST['thumb_h']), sanitize_text_field($_POST['thumb_zc']) ) )
						)
					);
				} // Is the Response
			} else {
				echo json_encode(array('error' => esc_html__('Invalid action send', 'wordpress-flat-countdown')));
			}

			wp_die();
		}

		public function wp_media_upload_image() {
			$image = wp_get_attachment_image_src( (int) sanitize_text_field($_REQUEST['att_id']), 'thumbnail' );
			die(json_encode(array(
				'status' 	=> 'valid',
				'thumb'		=> $image[0]
			)));
		}

		public function get_media_thumb() {
			$image = wp_get_attachment_image_src( (int) sanitize_text_field($_REQUEST['media_id']), array(180, 180) );
			die( json_encode( array(
				'thumb_url' => $image[0],
				'width' => $image[1],
				'height' => $image[2],
			) ) );
		}

		public function remove_gallery( $content) {
			return str_replace('[gallery]', '', $content);
		}

		/**
		* HTML escape given string
		*
		* @param string $text
		* @return string
		*/
		public function escape( $text) {
			$text = (string) $text;
			if ('' === $text) {
				return '';
			}

			$result = @htmlspecialchars($text, ENT_COMPAT, 'UTF-8');
			if (empty($result)) {
				$result = @htmlspecialchars(utf8_encode($text), ENT_COMPAT, 'UTF-8');
			}

			return $result;
		}

		public function multi_implode( $array, $glue) {
			$ret = '';

			foreach ($array as $item) {
				if (is_array($item)) {
					$ret .= $this->multi_implode($item, $glue) . $glue;
				} else {
					$ret .= $item . $glue;
				}
			}

			$ret = substr($ret, 0, 0-strlen($glue));

			return $ret;
		}

		/**
		 * Usefull
		 */

		//format right (for db insertion) php range function!
		public function doRange( $arr ) {
			$newarr = array();
			if ( is_array($arr) && count($arr)>0 ) {
				foreach ($arr as $k => $v) {
					$newarr[ $v ] = $v;
				}
			}
			return $newarr;
		}

		//verify if file exists!
		public function verifyFileExists( $file, $type = 'file') {
			clearstatcache();
			if ($type=='file') {
				if (!file_exists($file) || !is_file($file) || !is_readable($file)) {
					return false;
				}
				return true;
			} else if ($type=='folder') {
				if (!is_dir($file) || !is_readable($file)) {
					return false;
				}
				return true;
			}
			// invalid type
			return 0;
		}

		public function formatBytes( $bytes, $precision = 2) {
			$units = array('B', 'KB', 'MB', 'GB', 'TB');

			$bytes = max($bytes, 0);
			$pow = floor(( $bytes ? log($bytes) : 0 ) / log(1024));
			$pow = min($pow, count($units) - 1);

			// Uncomment one of the following alternatives
			// $bytes /= pow(1024, $pow);
			$bytes /= ( 1 << ( 10 * $pow ) );

			return round($bytes, $precision) . ' ' . $units[$pow];
		}

		public function prepareForInList( $v) {
			return "'" . $v . "'";
		}

		public function db_custom_insert( $table, $fields, $ignore = false, $wp_way = false) {
			if ( $wp_way && !$ignore ) {
				$this->db->insert(
					$table,
					$fields['values'],
					$fields['format']
				);
			} else {

				$formatVals = implode(', ', array_map(array('flatCountdown', 'prepareForInList'), $fields['format']));
				$theVals = array();
				foreach ( $fields['values'] as $k => $v ) {
					$theVals[] = $k;
				}

				$q = 'INSERT ' . ( $ignore ? 'IGNORE' : '' ) . " INTO $table (" . implode(', ', $theVals) . ') VALUES (' . $formatVals . ');';
				foreach ($fields['values'] as $kk => $vv) {
					$fields['values']["$kk"] = esc_sql($vv);
				}

				$q = vsprintf($q, $fields['values']);
				$r = $this->db->query( $q );
			}
		}

		/**
		 *
		 */
		/**
		 * setup module messages
		 */
		public function print_module_error( $module = array(), $error_number, $title = '' ) {
			$html = array();
			if ( count($module) == 0 ) {
				return true;
			}

			$html[] = '<div class="flatCountdown-grid_4 flatCountdown-error-using-module">';
			$html[] = 	'<div class="flatCountdown-panel">';
			$html[] = 		'<div class="flatCountdown-panel-header">';
			$html[] = 			'<span class="flatCountdown-panel-title">';
			$html[] = 				$title;
			$html[] = 			'</span>';
			$html[] = 		'</div>';
			$html[] = 		'<div class="flatCountdown-panel-content">';

			$error_msg = isset($module[$module['alias']]['errors'][$error_number]) ? $module[$module['alias']]['errors'][$error_number] : '';

			$html[] = 			'<div class="flatCountdown-error-details">' . ( $error_msg ) . '</div>';
			$html[] = 		'</div>';
			$html[] = 	'</div>';
			$html[] = '</div>';

			return implode("\n", $html);
		}

		public function convert_to_button( $button_params = array() ) {
			$button = array();
			$button[] = '<a';
			if (isset($button_params['url'])) {
				$button[] = ' href="' . ( $button_params['url'] ) . '"';
			}

			if (isset($button_params['target'])) {
				$button[] = ' target="' . ( $button_params['target'] ) . '"';
			}

			$button[] = ' class="flatCountdown-button';

			if (isset($button_params['color'])) {
				$button[] = ' ' . ( $button_params['color'] ) . '';
			}

			$button[] = '"';
			$button[] = '>';

			$button[] =  $button_params['title'];

			$button[] = '</a>';

			return implode('', $button);
		}

		public function load_terms( $taxonomy) {
			global $wpdb;

			$query = "SELECT DISTINCT t.name FROM {$wpdb->terms} AS t INNER JOIN {$wpdb->term_taxonomy} as tt ON tt.term_id = t.term_id WHERE 1=1 AND tt.taxonomy = '" . esc_sql($taxonomy) . "'";
			$result =  $wpdb->get_results($query , OBJECT);
			return $result;
		}

		/**
		 * flatCountdown related
		 */
		public function get_client_ip() {
			$ipaddress = '';

			if ($_SERVER['REMOTE_ADDR']) {
				$ipaddress = $_SERVER['REMOTE_ADDR'];
			} else if ($_SERVER['HTTP_CLIENT_IP']) {
				$ipaddress = $_SERVER['HTTP_CLIENT_IP'];
			} else if ($_SERVER['HTTP_X_FORWARDED']) {
				$ipaddress = $_SERVER['HTTP_X_FORWARDED'];
			} else if ($_SERVER['HTTP_FORWARDED_FOR']) {
				$ipaddress = $_SERVER['HTTP_FORWARDED_FOR'];
			} else if ( $_SERVER['HTTP_FORWARDED']) {
				$ipaddress = $_SERVER['HTTP_FORWARDED'];
			} else if ($_SERVER['HTTP_X_FORWARDED_FOR']) {
				$ipaddress = $_SERVER['HTTP_X_FORWARDED_FOR'];
			}

			return $ipaddress;
		}

		public function get_current_page_url() {
			$url = ( !empty($_SERVER['HTTPS']) )
				?
				'https://' . $_SERVER['SERVER_NAME'] . $_SERVER['REQUEST_URI']
				:
				'http://' . $_SERVER['SERVER_NAME'] . $_SERVER['REQUEST_URI']
			;
			return $url;
		}

		public function get_client_country() {
			$get_user_location = wp_remote_get( 'http://api.hostip.info/get_json.php?ip=' . $this->get_client_ip() );

			if ( is_wp_error( $get_user_location ) ) { // If there's error
				$body = false;
				// $err = htmlspecialchars( implode(';', $get_user_location->get_error_messages()) );
			} else {
				$body = wp_remote_retrieve_body( $get_user_location );
			}
			if (is_null($body) || !$body || trim($body)=='') { //status is Invalid!
				return false;
			} else {
				$body = json_decode($body);
				$user_country = array(
					'name'		=> $body->country_name,
					'code'		=> $body->country_code
				);
			}

			return $user_country;
		}

		// add_action( 'init', array($this, 'cookie_set') );
		public function cookie_set( $cookie_arr = array() ) {
			extract($cookie_arr);
			if ( !isset($_COOKIE['my_cookie']) ) {
				setcookie($name, $value, time()+$expire_sec);
			}
		}

		public function cookie_del( $cookie_arr = array() ) {
			extract($cookie_arr);
			setcookie($name, null, strtotime('-1 day'));
		}

		public function set_ajax_url() {
			$output  =	'<script type="text/javascript">';
			$output .=		'var ajaxurl = "' . admin_url('admin-ajax.php') . '";';
			$output .=	'</script>';
			echo $output;
		}

		public function print_widget_fields( $options = array(), $defaults = array() ) {
			$html = array();
			if ( count($options) > 0 ) {
				foreach ($options as $key => $value) {
					$val = '';
					if ( in_array( $key, array_keys($defaults) )) {
						$val = $defaults[$key];
					}
					if ( trim($val) == '' ) {
						$val = isset($value['std']) ? $value['std'] : '';
					}

					$html[] = '<p>';
					$html[] = 	'<label for="' . ( $key ) . '">' . ( $value['title'] ) . ':</label><br />';

					if ( $value['type'] == 'text' ) {
						$html[] = '<input class="widefat" ' . ( isset($value['width']) ? 'style="width:' . ( $value['width'] ) . '"' : '' ) . ' id="' . ( $key ) . '" name="' . ( $key ) . '" type="text" value="' . ( $val ) . '" />';
					} elseif ( $value['type'] == 'textarea' ) {
						$html[] = '<textarea class="widefat" ' . ( isset($value['width']) ? 'style="width:' . ( $value['width'] ) . '"' : '' ) . ' id="' . ( $key ) . '" name="' . ( $key ) . '">' . ( $val ) . '</textarea>';
					}
					$html[] = '</p>';
				}
			}

			return implode("\n", $html);
		}

		public function wp_pages() {
			$args = array(
				'sort_order' => 'ASC',
				'sort_column' => 'post_title',
				'hierarchical' => 1,
				'parent' => -1,
				'post_type' => 'page',
				'post_status' => 'publish'
			);
			$pages = get_pages($args);

			$ret_pages = array();
			if ( $pages && count($pages) > 0 ) {
				foreach ($pages as $page) {
					$ret_pages[$page->ID] = $page->post_title;
				}
			}

			return $ret_pages;
		}

		// displays error messages from form submissions
		public function flatCountdown_show_error_messages() {
			if ($codes = $this->flatCountdown_errors()->get_error_codes()) {
				echo '<div class="flatCountdown_errors">';
				foreach ($codes as $code) {
					 $message = $this->flatCountdown_errors()->get_error_message($code);
					 echo '<span class="flatCountdown-error"><strong>' . esc_html__('Error', 'wordpress-flat-countdown') . '</strong>: ' . $message . '</span><br/>';
				}
				echo '</div>';
			}
		}

		public function flatCountdown_errors() {
			static $wp_error;
			return isset($wp_error) ? $wp_error : ( $wp_error = new WP_Error(null, null, null) );
		}

		public function flatCountdown_getRealIpAddr() {
			if (!empty($_SERVER['HTTP_CLIENT_IP'])) {   //check ip from share internet
				$ip = $_SERVER['HTTP_CLIENT_IP'];
			} elseif (!empty($_SERVER['HTTP_X_FORWARDED_FOR'])) {   //to check ip is pass from proxy
				$ip = $_SERVER['HTTP_X_FORWARDED_FOR'];
			} else {
				$ip = $_SERVER['REMOTE_ADDR'];
			}
			return $ip;
		}

		public function flatCountdown_curPageURL() {
			return 'http://' . $_SERVER['SERVER_NAME'] . $_SERVER['REQUEST_URI'];
		}

		// Load frontpage
		public function flatCountdown_frontpage() {
			if ( !is_admin()) {
				// is ajax request
				if (
					isset($_REQUEST['email'])
					&& '' !== trim(sanitize_text_field($_REQUEST['email']))
					&& '' !== trim(sanitize_text_field($_REQUEST['name']))
					&& '' !== trim(sanitize_text_field($_REQUEST['sendnotifications']))
				) {
					// do ajax action
				} elseif (preg_match('/wp-login.php/', $_SERVER['SCRIPT_NAME'], $matches)) {
					// on login to admin
				} else {
					$home = true;
					$current_url = $this->flatCountdown_curPageURL();
					$rest = str_replace(home_url(), '', $current_url);
					if (strlen($rest) > 5) {
						$home = false;
					}
					if (isset( $_GET['page_id'] ) && sanitize_text_field(intval($_GET['page_id'])) > 0) {
						$home = false;
					}
					$countdownSettings = $this->getAllSettings();
					$countdownSettings = $countdownSettings['flatCountdown_config'];
					$development_ip = isset($countdownSettings['development_ip']) ? $countdownSettings['development_ip'] : '';
					if ( $countdownSettings['development_mode'] == 'yes'  ) {
						if (trim($development_ip) != '' && $development_ip == $this->flatCountdown_getRealIpAddr()) {
							// load home page
							if ($home) {
								require_once( $this->cfg['paths']['templates_dir_path'] . '/index.php' );
							} else {
								require_once( $this->cfg['paths']['templates_dir_path'] . '/page.php' );
							}
						}
					} else {
						// load home page
						if ($home) {
							require_once( $this->cfg['paths']['templates_dir_path'] . '/index.php' );
						} else {
							if (isset( $_GET['page_id'] ) && sanitize_text_field(intval($_GET['page_id'])) > 0) {
								require_once( $this->cfg['paths']['templates_dir_path'] . '/page.php' );
							} elseif (isset( $_GET['p'] ) && sanitize_text_field(intval($_GET['p'])) > 0) {
								require_once( $this->cfg['paths']['templates_dir_path'] . '/page.php' );
							}

						}
					}
				}
			}
		}

		public function flatCountdownSubscribe() {
			global $wpdb;

			if ( isset( $_REQUEST['action'] ) &&  'flatCountdownSubscribe' === sanitize_text_field($_REQUEST['action']) ) {
				parse_str($_REQUEST['formFields']);

				if ( $name == '' ) {
					die(json_encode(array(
						'status' => 'error',
						'html' => esc_html__('Error!: Name field empty!', 'wordpress-flat-countdown')
					)));
				}

				if ( $email == '' ) {
					die(json_encode(array(
						'status' => 'error',
						'html' => esc_html__('Error!: Email field empty!', 'wordpress-flat-countdown')
					)));
				}

				//$email = uniqid() . $email;
				//if( !eregi( "^[_a-z0-9-]+(\.[_a-z0-9-]+)*@[a-z0-9-]+(\.[a-z0-9-]+)*(\.[a-z]{2,3})$", $email ) ) {
				if (!filter_var($email, FILTER_VALIDATE_EMAIL)) {
					die(json_encode(array(
						'status' => 'error',
						'html' => esc_html__('Error!: Email invalid!', 'wordpress-flat-countdown')
					)));
				}

				$email_results = $wpdb->get_results( 'SELECT * FROM ' . $wpdb->prefix . 'wp_flat_coundown WHERE email = "' . $email . '"', OBJECT );

				if ( count( $email_results ) > 0 ) {
					die(json_encode(array(
						'status' => 'error',
						'html' => esc_html__('Error!: Email is already registered!', 'wordpress-flat-countdown')
					)));
				}

				if ( $cptch != $security_code ) {
					die(json_encode(array(
						'status' => 'error',
						'html' => esc_html__('Error!: Invalid Captcha Code!', 'wordpress-flat-countdown')
					)));
				}

				if ( isset( $name ) && $name != '' && isset( $email ) && $email != '' ) {
					$wpdb->insert($wpdb->prefix . 'wp_flat_coundown', array(
						'name' => $name,
						'email' => $email,
						'sendnotifications' => $send_notification
					));

					if ( $send_notification == 'on' ) {
						$this->sendEmail('user', $email, $name);
					}

					$this->sendEmail('admin', $email, $name);

					die(json_encode(array(
						'status' => 'ok',
						'html' => esc_html__('Success!', 'wordpress-flat-countdown')
					)));
				} else {
					die(json_encode(array(
						'status' => 'error',
						'html' => esc_html__('Error!', 'wordpress-flat-countdown')
					)));
				}
			}
		}

		public function template_vars_replacement( $template, $user_email = '', $user_name = '' ) {

			$to_replace = array(
				'blog_url'         => get_option( 'siteurl' ),
				'home_url'         => home_url(), // get_option( 'home' ),
				'blog_name'        => get_option( 'blogname' ),
				'blog_description' => get_option( 'blogdescription' ),
				'admin_email'      => get_option( 'admin_email' ),
				'date'             => date_i18n( get_option( 'date_format' ) ),
				'time'             => date_i18n( get_option( 'time_format' ) ),
				'user_email'	   => $user_email,
				'user_name'        => $user_name
			);
			$to_replace = apply_filters( 'flatCountdown_tags', $to_replace );

			foreach ( $to_replace as $tag => $var ) {
				$template = str_replace( '%' . $tag . '%', $var, $template );
			}
			return $template;
		}

		public function sendEmail( $user_type, $user_email = '', $user_name = '' ) {
			$settings = $this->getAllSettings();
			$settings = $settings['flatCountdown_config'];
			if ( $user_type == 'admin' ) {
				$notification_email = $settings['admin_notification_email_body'];
				$sendto = $settings['notification_emails'];
				$subj = $this->template_vars_replacement( $settings['admin_notification_subject'], '', '');
				$from = get_option( 'blogname' );
			} elseif ( $user_type == 'user' ) {
				$notification_email = $settings['user_notification_email_body'];
				$sendto = $user_email;
				$subj = $this->template_vars_replacement( $settings['user_notification_subject'], '', '');
				$from = $this->template_vars_replacement( $settings['user_email_from'], '', '' );
			}

			$multiple_recipients = array( $sendto );

			//$body = $user_notification_email;
			$body = '
			<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
				<html>
					<head>
						<meta http-equiv="Content-Type" content="text/html;' . get_option('blog_charset') . '" />
					</head>
					<body style="margin: 0px; background-color: #F4F3F4; font-family: Helvetica, Arial, sans-serif; font-size:12px;" text="#444444" bgcolor="#F4F3F4" link="#21759B" alink="#21759B" vlink="#21759B" marginheight="0" topmargin="0" marginwidth="0" leftmargin="0">
						<table cellpadding="0" cellspacing="0" width="100%" bgcolor="#F4F3F4" border="0">
							<tr>
								<td style="padding:15px;">
									<center>
										<table width="550" cellpadding="0" bgcolor="#ffffff" cellspacing="0" align="center">
											<tr>
												<td align="left">
													<div style="border:solid 1px #d9d9d9;">
														<table id="header" width="100%" border="0" cellpadding="0" bgcolor="#ffffff" cellspacing="0" style="line-height:1.6;font-size:12px;font-family: Helvetica, Arial, sans-serif;border:solid 1px #FFFFFF;color:#444;">
															<tr>
																<td colspan="2" height="30" style="color:#ffffff;" valign="bottom">.</td>
															</tr>
															<tr>
																<td style="line-height:32px;padding-left:30px;" valign="baseline"><span style="font-size:32px;"><a href="%blog_url%" style="text-decoration:none;" target="_blank">%blog_name%</a></span></td>
																<td style="padding-right:30px;" align="right" valign="baseline"><span style="font-size:14px;color:#777777">%blog_description%</span></td>
															</tr>
														</table>
														<table id="content" width="490" border="0" cellpadding="0" bgcolor="#ffffff" cellspacing="0" style="margin-top:15px;margin-right:30px; margin-left:30px;color:#444;line-height:1.6;font-size:12px;font-family: Arial, sans-serif;color: #444;">
															<tr>
																<td colspan="2" style="border-top: solid 1px #d9d9d9">
																	<div style="padding:15px 0;">
																		' . $notification_email . '
																	</div>
																</td>
															</tr>
														</table>
														<table id="footer" width="490" border="0" cellpadding="0" bgcolor="#ffffff" cellspacing="0" style="line-height:1.5;font-size:12px;font-family: Arial, sans-serif;margin-right:30px;margin-left:30px;">
															<tr style="font-size:11px;color:#999999;">
																<td style="border-top: solid 1px #d9d9d9;" colspan="2">
																	<div style="padding-top:15px; padding-bottom:1px;">' . esc_html__('Email sent', 'wordpress-flat-countdown') . ' %date% @ %time%</div>
																	<div>' . esc_html__('For any requests, please contact', 'wordpress-flat-countdown') . ' <a href="mailto:%admin_email%">%admin_email%</a></div>
																</td>
															</tr>
						                                    <tr>
																<td colspan="2" style="color:#ffffff;" height="15">.</td>
						                                    </tr>
														</table>
													</div>
												</td>
											</tr>
										</table>
									</center>
								</td>
							</tr>
						</table>
					</body>
				</html>';

			$body = apply_filters( 'flatCountdown_plaintext_body', $this->template_vars_replacement( $body, $user_email, $user_name ) );

			$to = $multiple_recipients;

			$subject = $subj;
			$headers = 'From: ' . $from . "\r\n";
			$headers .= 'Reply-To: ' . get_option( 'admin_email' ) . "\r\n";
			$headers = "MIME-Version: 1.0\r\n";
			$headers .= "Content-Type: text/html; charset=ISO-8859-1\r\n";
			//$sendto = 'alexandra.ipate@gmail.com';
			mail( $sendto, $subject, $body, $headers );

			//echo $sendto;

			/*die(json_encode(array(
				'status' => 'ok',
				'html' => esc_html__('Email notification sent!', 'wordpress-flat-countdown')
			)));*/

		}

		public function sendTestEmail() {
			$this->sendEmail('admin');
		}

		public function saveSubscribers() {
			global $flatCountdown, $wpdb;

			header('Content-Type: text/csv; charset=utf-8');
			header('Content-Disposition: attachment; filename=subscribers.csv');
			$output = fopen('php://output', 'w');

			fputcsv($output, array('Subscribe ID', 'Name', 'Email', 'Send Notifications'));

			$subscribers_table = $wpdb->prefix . 'wp_flat_coundown';
			$subscribers = $wpdb->get_results( 'SELECT * FROM ' . $subscribers_table, ARRAY_A );

			foreach ( $subscribers as $subscriber ) {
				fputcsv( $output, $subscriber );
			}

		}

		public function getAllGfonts( $what = 'all' ) {
			$fonts = json_decode(
				//$this->the_theme->wp_filesystem->get_contents( $this->the_theme->cfg['paths']['theme_dir_path'] . '/fonts/google-webfonts.json' )
			//, true);
				file_get_contents( $this->cfg['paths']['freamwork_dir_path'] . '/fonts/google-webfonts.json' )
			, true);

			$ret_fonts = array();
			if (count($fonts['items']) > 0 ) {
				foreach ( $fonts['items'] as $font ) {
					$ret_fonts[$font['family']] = $font['family'];
				}
			}

			if ( $what == 'all' ) {
				return $ret_fonts;
			}

		}

		public function hex2rgb( $hex) {
			$hex = str_replace('#', '', $hex);

			if (strlen($hex) == 3) {
				$r = hexdec(substr($hex, 0, 1) . substr($hex, 0, 1));
				$g = hexdec(substr($hex, 1, 1) . substr($hex, 1, 1));
				$b = hexdec(substr($hex, 2, 1) . substr($hex, 2, 1));
			} else {
				$r = hexdec(substr($hex, 0, 2));
				$g = hexdec(substr($hex, 2, 2));
				$b = hexdec(substr($hex, 4, 2));
			}
			$rgb = array($r, $g, $b);
			//return implode(",", $rgb); // returns the rgb values separated by commas
			return $rgb; // returns an array with the rgb values
		}

	}
}
