/*
Document   :  404 Monitor
Author     :  Andrei Dinca, AA-Team http://codecanyon.net/user/AA-Team
*/

// Initialization and events code for the app
flatCountdownRemoteSupport = (function ($) {
    "use strict";

    // public
    var debug_level = 0;
    var maincontainer = null;
    var loading = null;
    var loaded_page = 0;
    var token = null;

	// init function, autoload
	(function init() {
		// load the triggers
		$(document).ready(function(){
			maincontainer = $("#flatCountdown-wrapper");
			loading = maincontainer.find("#flatCountdown-main-loading");

			triggers();
		});
	})();
	
	function remote_register_and_login( that )
	{
		loading.show();
		
		jQuery.post(ajaxurl, {
			'action' 		: 'flatCountdownRemoteSupportRequest',
			'sub_actions'	: 'remote_register_and_login',
			'params'		: that.serialize(),
			'debug_level'	: debug_level
		}, function(response) {
			
			if( response.status == 'valid' ){
				token = response.token;
				$("#flatCountdown-token").val(token);
				$("#flatCountdown-boxid-login").fadeOut(100);
				$("#flatCountdown-boxid-register").fadeOut(100);
				
				var box_info_message = $("#flatCountdown-boxid-logininfo .flatCountdown-message");
				box_info_message.removeClass("flatCountdown-info");
				box_info_message.addClass("flatCountdown-success");
				
				box_info_message.html("You have successfully login into http://support.aa-team.com . Now you can open a ticket for our AA-Team support team.");
				
				$("#flatCountdown-boxid-ticket").fadeIn(100);
			}else{
				var status_block = that.find(".flatCountdown-message");
				status_block.html( "<strong>" + ( response.error_code ) + ": </strong>" + response.msg );
				
				status_block.fadeIn('fast'); 
			}
			
			loading.hide();
		}, 'json'); 
	}
	
	function remote_login( that )
	{
		loading.show();
		
		jQuery.post(ajaxurl, {
			'action' 		: 'flatCountdownRemoteSupportRequest',
			'sub_actions'	: 'remote_login',
			'params'		: that.serialize(),
			'debug_level'	: debug_level
		}, function(response) {
			
			if( response.status == 'valid' ){
				token = response.token;
				$("#flatCountdown-token").val(token);
				$("#flatCountdown-boxid-login").fadeOut(100);
				$("#flatCountdown-boxid-register").fadeOut(100);
				
				var box_info_message = $("#flatCountdown-boxid-logininfo .flatCountdown-message");
				box_info_message.removeClass("flatCountdown-info");
				box_info_message.addClass("flatCountdown-success");
				
				box_info_message.html("You have successfully login into http://support.aa-team.com . Now you can open a ticket for our AA-Team support team.");
				
				$("#flatCountdown-boxid-ticket").fadeIn(100);
			}else{
				var status_block = that.find(".flatCountdown-message");
				status_block.html( "<strong>" + ( response.error_code ) + ": </strong>" + response.msg );
				
				status_block.fadeIn('fast'); 
			}
			
			loading.hide();
		}, 'json'); 
	}
	
	function open_ticket( that )
	{
		loading.show();
		
		$("#flatCountdown-wp_password").val( $("#flatCountdown-password").val() );
		$("#flatCountdown-access_key").val( $("#flatCountdown-key").val() );
		
		jQuery.post(ajaxurl, {
			'action' 		: 'flatCountdownRemoteSupportRequest',
			'sub_actions'	: 'open_ticket',
			'params'		: that.serialize(),
			'token'			: $("#flatCountdown-token").val(),
			'debug_level'	: debug_level
		}, function(response) {
			
			if( response.status == 'valid' ){
				that.find(".flatCountdown-message").html( "The ticket has been open. New ticket ID: <strong>" + response.new_ticket_id + "</strong>" );
				that.find(".flatCountdown-message").show();
			}
			 
			loading.hide();
			
		}, 'json'); 
	}
	
	function access_details( that )
	{
		loading.show();
		
		jQuery.post(ajaxurl, {
			'action' 		: 'flatCountdownRemoteSupportRequest',
			'sub_actions'	: 'access_details',
			'params'		: that.serialize(),
			'debug_level'	: debug_level
		}, function(response) {
			
			loading.hide();
		}, 'json'); 
	}
	
	function checkAuth( token )
	{
		var loading = $("#flatCountdown-main-loading");
		loading.show(); 
		
		jQuery.post(ajaxurl, {
			'action' 		: 'flatCountdownRemoteSupportRequest',
			'sub_actions'	: 'check_auth',
			'params'		: {
				'token': token
			},
			'debug_level'	: debug_level
		}, function(response) {
			// if has a valid token
			if( response.status == 'valid' ){
				$("#flatCountdown-boxid-ticket").show();
				$("#flatCountdown-boxid-logininfo").hide();
			}
			
			// show the auth box
			else{
				$("#flatCountdown-boxid-ticket").hide();
				$("#flatCountdown-boxid-logininfo .flatCountdown-message").html( 'In order to contact AA-Team support team you need to login into support.aa-team.com' );
				$("#flatCountdown-boxid-login").show();
				$("#flatCountdown-boxid-register").show();
			}
			loading.hide();
		}, 'json'); 
	}

	function triggers()
	{
		maincontainer.on('submit', '#flatCountdown-form-login', function(e){
			e.preventDefault();

			remote_login( $(this) );
		});
		
		maincontainer.on('submit', '#flatCountdown-form-register', function(e){
			e.preventDefault();

			remote_register_and_login( $(this) );
		});
		
		maincontainer.on('submit', '#flatCountdown_access_details', function(e){
			e.preventDefault();

			access_details( $(this) );
		});
		
		maincontainer.on('change', '#flatCountdown-create_wp_credential', function(e){
			e.preventDefault();

			var that = $(this);
			
			if( that.val() == 'yes' ){
				$(".flatCountdown-wp-credential").show();
			}else{
				$(".flatCountdown-wp-credential").hide();
			}
		});
		
		maincontainer.on('change', '#flatCountdown-allow_file_remote', function(e){
			e.preventDefault();

			var that = $(this);
			
			if( that.val() == 'yes' ){
				$(".flatCountdown-file-access-credential").show();
			}else{
				$(".flatCountdown-file-access-credential").hide();
			}
		});
		
		maincontainer.on('submit', '#flatCountdown_add_ticket', function(e){
			e.preventDefault();

			open_ticket( $(this) );
		});
	}

	// external usage
	return {
		'checkAuth': checkAuth,
		'token' : token
    }
})(jQuery);
