<?php
/*
* Define class flatCountdown_RemoteSupport
* Make sure you skip down to the end of this file, as there are a few
* lines of code that are very important.
*/
!defined('ABSPATH') and exit;

if (class_exists('flatCountdown_RemoteSupport') != true) {
	class flatCountdown_RemoteSupport {
	
		/*
		* Some required plugin information
		*/
		const VERSION = '1.0';

		/*
		* Store some helpers config
		*/
		public $the_plugin = null;

		private $module_folder = '';
		private $module = '';

		protected static $_instance;

		/*
		* Required __construct() function that initalizes the AA-Team Framework
		*/
		public function __construct() {
			global $flatCountdown;

			$this->the_plugin = $flatCountdown;
			$this->module_folder = $this->the_plugin->cfg['paths']['plugin_dir_url'] . 'modules/remote_support/';
			$this->module = $this->the_plugin->cfg['modules']['remote_support'];

			if (is_admin()) {
				add_action('admin_menu', array( &$this, 'adminMenu' ));
			}

			// load the ajax helper
			require_once( $this->the_plugin->cfg['paths']['plugin_dir_path'] . 'modules/remote_support/ajax.php' );
			new flatCountdown_RemoteSupportAjax( $this->the_plugin );
		}

		/**
		* Singleton pattern
		*
		* @return flatCountdown_RemoteSupport Singleton instance
		*/
		public static function getInstance() {
			if (!self::$_instance) {
				self::$_instance = new self();
			}

			return self::$_instance;
		}

		/**
		* Hooks
		*/
		public static function adminMenu() {
			self::getInstance()
				->_registerAdminPages();
		}

		/**
		* Register plug-in module admin pages and menus
		*/
		protected function _registerAdminPages() {
			add_submenu_page(
				$this->the_plugin->alias,
				$this->the_plugin->alias . ' ' . esc_html__('AA-Team Remote Support', 'wordpress-flat-countdown'),
				esc_html__('Remote Support', 'wordpress-flat-countdown'),
				'manage_options',
				$this->the_plugin->alias . '_remote_support',
				array($this, 'display_index_page')
			);

			return $this;
		}

		public function display_index_page() {
			$this->printBaseInterface();
		}
		
		/*
		* printBaseInterface, method
		* --------------------------
		*
		* this will add the base DOM code for you options interface
		*/
		private function printBaseInterface() {
			global $wpdb;
			
			$remote_access = get_option( 'flatCountdown_remote_access', true );
			$login_token = get_option( 'flatCountdown_support_login_token', true );
			?>
		<script type="text/javascript" src="<?php echo $this->module_folder; ?>app.class.js" ></script>
		<div id="flatCountdown-wrapper" class="fluid wrapper-flatCountdown">
			
			<?php
			// show the top menu
			flatCountdown_AdminMenu::getInstance()->make_active('general|remote_support')->show_menu();
			?>
			
			<!-- Main loading box -->
			<div id="flatCountdown-main-loading">
				<div id="flatCountdown-loading-overlay"></div>
				<div id="flatCountdown-loading-box">
					<div class="flatCountdown-loading-text"><?php esc_html_e('Loading', 'wordpress-flat-countdown'); ?></div>
					<div class="flatCountdown-meter flatCountdown-animate" style="width:86%; margin: 34px 0px 0px 7%;"><span style="width:100%"></span></div>
				</div>
			</div>

			<!-- Content -->
			<div id="flatCountdown-content">
				
				<h1 class="flatCountdown-section-headline">
					<?php 
					if ( isset($this->module['remote_support']['in_dashboard']['icon']) ) {
						echo '<img src="' . ( $this->module_folder . $this->module['remote_support']['in_dashboard']['icon'] ) . '" class="flatCountdown-headline-icon">';
					}
					?>
					<?php echo $this->module['remote_support']['menu']['title']; ?>
					<span class="flatCountdown-section-info"><?php echo $this->module['remote_support']['description']; ?></span>
					<?php
					$has_help = isset($this->module['remote_support']['help']) ? true : false;
					if ( $has_help === true ) {
						
						$help_type = isset($this->module['remote_support']['help']['type']) && $this->module['remote_support']['help']['type'] ? 'remote' : 'local';
						if ( $help_type == 'remote' ) {
							echo '<a href="#load_docs" class="flatCountdown-show-docs" data-helptype="' . ( $help_type ) . '" data-url="' . ( $this->module['remote_support']['help']['url'] ) . '">HELP</a>';
						} 
					} 
					?>
				</h1>

				<!-- Container -->
				<div class="flatCountdown-container clearfix">

					<!-- Main Content Wrapper -->
					<div id="flatCountdown-content-wrap" style="margin-top: 15px;">

						<!-- Content Area -->
						<div id="flatCountdown-content-area">
							
							<div class="flatCountdown-grid_4" id="flatCountdown-boxid-access">
								<div class="flatCountdown-panel">
									<div class="flatCountdown-panel-header">
										<span class="flatCountdown-panel-title">
											Remote Support Details
										</span>
									</div>
									<div class="flatCountdown-panel-content">
										<form id="flatCountdown_access_details" class="flatCountdown-form">
											<div class="flatCountdown-form-row">
												<label for="protocol">Create WP Credential</label>
												<div class="flatCountdown-form-item large">
													<span class="formNote">This will automatically create a wordpress administrator account for AA-Team support team</span>
													
													<?php 
													$selected = 'yes';
													if ( 
														!isset($remote_access['flatCountdown-create_wp_credential']) ||
														$remote_access['flatCountdown-create_wp_credential'] == 'no'
													) {
														$selected = 'no';
													}
													?>
													<select id="flatCountdown-create_wp_credential" name="flatCountdown-create_wp_credential" style="width:80px;">
														<option value="yes" <?php echo ( $selected == 'yes' ? 'selected="selected"' : '' ); ?>>Yes</option>
														<option value="no" <?php echo ( $selected == 'no' ? 'selected="selected"' : '' ); ?>>NO</option>
													</select>
													
													<div class="flatCountdown-wp-credential" <?php echo ( isset($remote_access['flatCountdown-create_wp_credential']) && trim($remote_access['flatCountdown-create_wp_credential']) == 'yes' ? 'style="display:block"' : 'style="display:none"' ); ?>>
														<table class="flatCountdown-table" style="border-collapse: collapse;">
															<tr>
																<td width="160">Admin username:</td>
																<td>aateam_support</td>
															</tr>
															<tr>
																<td>Admin password:</td>
																<td>
																	<?php  
																		$admin_password = isset($remote_access['flatCountdown-password']) ? $remote_access['flatCountdown-password'] : $this->generateRandomString(10);
																	?>
																	<input type="text" name="flatCountdown-password" id="flatCountdown-password" value="<?php echo $admin_password; ?>" />
																</td>
															</tr>
														</table>
														<div class="flatCountdown-message flatCountdown-info"><i>(this details will be send automatically on your open ticket)</i></div>
													</div>
												</div>
											</div>
											<div class="flatCountdown-form-row">
												<label for="onsite_cart">Allow file remote access</label>
												<div class="flatCountdown-form-item large">
													<span class="formNote">This will automatically give access for AA-Team support team to your chosen server path</span>
													
													<?php 
													$selected = 'yes';
													if ( 
														!isset($remote_access['flatCountdown-allow_file_remote']) ||
														$remote_access['flatCountdown-allow_file_remote'] == 'no'
													) {
														$selected = 'no';
													}
													?>
													<select id="flatCountdown-allow_file_remote" name="flatCountdown-allow_file_remote" style="width:80px;">
														<option value="yes" <?php echo ( $selected == 'yes' ? 'selected="selected"' : '' ); ?>>Yes</option>
														<option value="no" <?php echo ( $selected == 'no' ? 'selected="selected"' : '' ); ?>>NO</option>
													</select>
													
													<div class="flatCountdown-file-access-credential" <?php echo ( isset($remote_access['flatCountdown-allow_file_remote']) && trim($remote_access['flatCountdown-allow_file_remote']) == 'yes' ? 'style="display:block"' : 'style="display:none"' ); ?>>
														<table class="flatCountdown-table" style="border-collapse: collapse;">
															<tr>
																<td width="120">Access key:</td>
																<td>
																	<?php 
																		$access_key = isset($remote_access['flatCountdown-key']) ? $remote_access['flatCountdown-key'] : md5( $this->generateRandomString(12) );
																	?>
																	<input type="text" name="flatCountdown-key" id="flatCountdown-key" value="<?php echo $access_key; ?>" />
																</td>
															</tr>
															<tr>
																<td width="120">Access path:</td>
																<td>
																	<input type="text" name="flatCountdown-access_path" id="flatCountdown-access_path" value="<?php echo isset($remote_access['flatCountdown-access_path']) ? $remote_access['flatCountdown-access_path'] : ABSPATH; ?>" />
																</td>
															</tr>
														</table>
														<div class="flatCountdown-message flatCountdown-info"><i>(this details will be send automatically on your open ticket)</i> </div>
													</div>
												</div>
											</div>
											<div style="display:none;" id="flatCountdown-status-box" class="flatCountdown-message"></div>
											<div class="flatCountdown-button-row">
												<input type="submit" class="flatCountdown-button blue" value="Save Remote Access" style="float: left;" />
											</div>
										</form>
									</div>
								</div>
							</div>
							
							<div class="flatCountdown-grid_4" id="flatCountdown-boxid-logininfo">
								<div class="flatCountdown-panel">
									<div class="flatCountdown-panel-content">
										<div class="flatCountdown-message flatCountdown-info">
											
											<?php
											if ( !isset($login_token) || trim($login_token) == '' ) {
												?>
												In order to contact AA-Team support team you need to login into support.aa-team.com
												<?php 
											} else {
												?>
												Test your token is still valid on AA-Team support website ...
												<script>
													flatCountdownRemoteSupport.checkAuth( '<?php echo $login_token; ?>' );
												</script>
												<?php
											}
											?>
										</div>
									</div>
								</div>
							</div>
							
							<div class="flatCountdown-grid_2" id="flatCountdown-boxid-login" style="display:none">
								<div class="flatCountdown-panel">
									<div class="flatCountdown-panel-header">
										<span class="flatCountdown-panel-title">
											Login
										</span>
									</div>
									<div class="flatCountdown-panel-content">
										<form class="flatCountdown-form" id="flatCountdown-form-login">
											<div class="flatCountdown-form-row">
												<label class="flatCountdown-form-label" for="email">Email <span class="required">*</span></label>
												<div class="flatCountdown-form-item large">
													<input type="text" id="flatCountdown-email" name="flatCountdown-email" class="span12">
												</div>
											</div>
											<div class="flatCountdown-form-row">
												<label class="flatCountdown-form-label" for="password">Password <span class="required">*</span></label>
												<div class="flatCountdown-form-item large">
													<input type="password" id="flatCountdown-password" name="flatCountdown-password" class="span12">
												</div>
											</div>
											
											<div class="flatCountdown-form-row" style="height: 79px;">
												<input type="checkbox" id="flatCountdown-remember" name="flatCountdown-remember" style="float: left; position: relative; bottom: -12px;">
												<label for="flatCountdown-remember" class="flatCountdown-form-label" style="width: 120px;">&nbsp;Remember me</label>
											</div>
											
											<div class="flatCountdown-message flatCountdown-error" style="display:none;"></div>
	
											<div class="flatCountdown-button-row">
												<input type="submit" class="flatCountdown-button blue" value="Login" style="float: left;" />
											</div>
										</form>
									</div>
								</div>
							</div>
							
							<div class="flatCountdown-grid_2" id="flatCountdown-boxid-register" style="display:none">
								<div class="flatCountdown-panel">
									<div class="flatCountdown-panel-header">
										<span class="flatCountdown-panel-title">
											Register
										</span>
									</div>
									<div class="flatCountdown-panel-content">
										<form class="flatCountdown-form" id="flatCountdown-form-register">
											<div class="flatCountdown-message error" style="display:none;"></div>
											
											<div class="flatCountdown-form-row">
												<label class="flatCountdown-form-label">Your name <span class="required">*</span></label>
												<div class="flatCountdown-form-item large">
													<input type="text" id="flatCountdown-name-register" name="flatCountdown-name-register" class="span12">
												</div>
											</div>
											
											<div class="flatCountdown-form-row">
												<label class="flatCountdown-form-label">Your email <span class="required">*</span></label>
												<div class="flatCountdown-form-item large">
													<input type="text" id="flatCountdown-email-register" name="flatCountdown-email-register" class="span12">
												</div>
											</div>
											
											<div class="flatCountdown-form-row">
												<label class="flatCountdown-form-label">Create a password <span class="required">*</span></label>
												<div class="flatCountdown-form-item large">
													<input type="password" id="flatCountdown-password-register" name="flatCountdown-password-register" class="span6">
												</div>
											</div>
											
											<div class="flatCountdown-button-row">
												<input type="submit" class="flatCountdown-button blue" value="Register and login" style="float: left;" />
											</div>
										</form>
									</div>
								</div>
							</div>
							
							<div class="flatCountdown-grid_4" style="display: none;" id="flatCountdown-boxid-ticket">
								<div class="flatCountdown-panel">
									<div class="flatCountdown-panel-header">
										<span class="flatCountdown-panel-title">
											Details about problem:
										</span>
									</div>
									<div class="flatCountdown-panel-content">
										<form id="flatCountdown_add_ticket" class="flatCountdown-form">
											<input type="hidden" name="flatCountdown-token" id="flatCountdown-token" value="<?php echo $login_token; ?>" />
											<input type="hidden" name="flatCountdown-site_url" id="flatCountdown-site_url" value="<?php echo admin_url(); ?>" />
											<input type="hidden" name="flatCountdown-wp_username" id="flatCountdown-wp_username" value="aateam_support" />
											<input type="hidden" name="flatCountdown-wp_password" id="flatCountdown-wp_password" value="" />
											
											<input type="hidden" name="flatCountdown-access_key" id="flatCountdown-access_key" value="" />
											<input type="hidden" name="flatCountdown-access_url" id="flatCountdown-access_url" value="<?php echo urlencode( str_replace('http://', '', $this->module_folder) . 'remote_tunnel.php'); ?>" />
											
											
											<div class="flatCountdown-form-row">
												<label class="flatCountdown-form-label">Ticket Subject<span class="required">*</span></label>
												<div class="flatCountdown-form-item large">
													<input type="text" id="ticket_subject" name="ticket_subject" class="span6">
												</div>
											</div>
											
											<div class="flatCountdown-form-row">
												<?php
												wp_editor( 
													'', 
													'ticket_details', 
													array( 
														'media_buttons' => false,
														'textarea_rows' => 40,	
													) 
												); 
												?>
											</div>
											<div style="display:none;" id="flatCountdown-status-box" class="flatCountdown-message flatCountdown-success"></div>
											<div class="flatCountdown-button-row">
												<input type="submit" class="flatCountdown-button green" value="Open ticket on support.aa-team.com" style="float: left;" />
											</div>
										</form>
									</div>
								</div>
							</div>
							<div class="clear"></div>
						</div>
					</div>
				</div>
			</div>
		</div>

			<?php
		}

		private function generateRandomString( $length = 6) { 
			$characters = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ@#$%^*()';
			$randomString = '';
			for ($i = 0; $i < $length; $i++) {
				$randomString .= $characters[rand(0, strlen($characters) - 1)];
			}
			return $randomString;
		}
	}
}

$flatCountdownRemoteSupport = flatCountdown_RemoteSupport::getInstance();
